@@ -1,6 +1,6 @@
 package Pithub::Base;
 {
-  $Pithub::Base::VERSION = '0.01017';
+  $Pithub::Base::VERSION = '0.01018';
 }
 
 # ABSTRACT: Github v3 base class for all Pithub modules
@@ -66,7 +66,7 @@ has 'repo' => (
 has 'token' => (
     clearer   => 'clear_token',
     is        => 'rw',
-    predicate => 'has_token',
+    predicate => '_has_token',
     required  => 0,
 );
 
@@ -216,11 +216,12 @@ sub request {
 
     my $uri = $self->_uri_for($path);
 
-    if ( $self->_token_required( $method, $path ) && !$self->has_token ) {
+    my $request = $self->_request_for( $method, $uri, $data );
+
+    if ( $self->_token_required( $method, $path ) && !$self->has_token($request) ) {
         croak sprintf "Access token required for: %s %s (%s)", $method, $path, $uri;
     }
 
-    my $request = $self->_request_for( $method, $uri, $data );
 
     if ($options) {
         croak 'The key options must be a hashref' unless ref $options eq 'HASH';
@@ -245,6 +246,19 @@ sub request {
     );
 }
 
+
+sub has_token {
+    my ($self, $request) = @_;
+
+    # If we have one specified in the object, return true
+    return 1 if $self->_has_token;
+    # If no reqest object here, we don't have a token
+    return 0  unless $request;
+
+    return 1 if $request->header('Authorization');
+    return 0;
+}
+
 sub _build__json {
     my ($self) = @_;
     return JSON::Any->new;
@@ -366,7 +380,7 @@ Pithub::Base - Github v3 base class for all Pithub modules
 
 =head1 VERSION
 
-version 0.01017
+version 0.01018
 
 =head1 DESCRIPTION
 
@@ -783,6 +797,11 @@ Same as L<Pithub::GitData::Trees/get>:
 
 This method always returns a L<Pithub::Result> object.
 
+=head2 has_token (?$request)
+
+This method checks if a token has been specified, or if not, and a request
+object is passed, then it looks for an Authorization header in the request.
+
 =head1 AUTHOR
 
 Johannes Plunien <plu@cpan.org>
